#include "internal.h"
#include "geo.h"


octGetPoints(object, num_points, points)
octObject *object;
int32 *num_points;
struct octPoint *points;
{
    generic *ptr;
    struct geo_points *gp;
    int i;
    int32 flags, num_pts;

    ptr = oct_id_to_ptr(object->objectId);
    if (ptr == NIL(generic)) {
	oct_error("OctGetPoints: The object's id has been corrupted");
	return OCT_CORRUPTED_OBJECT;
    }

    flags = oct_object_descs[ptr->flags.type].flags;
    if (!(flags&HAS_POINTS)) {
	oct_error("octGetPoints: You can not get points from a %s",
		  oct_type_names[ptr->flags.type]);
	return OCT_ERROR;
    }

    gp = (struct geo_points *) ptr;

    num_pts = gp->num_points;

    if (points != NIL(struct octPoint)) {
	struct octPoint *pt_ptr = gp->points;
	if (*num_points < num_pts) {
	    *num_points = num_pts;
	    oct_error(
		   "octGetPoints: the array is too small to hold the points");
	    return OCT_TOO_SMALL;
	}
	for (i = num_pts; i > 0; i--) {
	    *(points++) = *(pt_ptr++);
	}
    }
    *num_points = num_pts;
    
    return OCT_OK;
}

octPutPoints(object, num_points, points)
octObject *object;
int32 num_points;
struct octPoint *points;
{
    generic *ptr;
    struct geo_points *gp;
    struct octBox old_box, new_box;
    struct object_desc *object_desc;
    int32 flags, old_num_points;
    int i, record;
    struct octPoint *pt_ptr, *head, *old_points;

    ptr = oct_id_to_ptr(object->objectId);
    if (ptr == NIL(generic)) {
	oct_error("OctPutPoints: The object's id has been corrupted");
	return OCT_CORRUPTED_OBJECT;
    }

    object_desc = &oct_object_descs[ptr->flags.type];
    flags = object_desc->flags;

    if (!flags&HAS_POINTS) {
	oct_error("octPutPoints: You can not put points on a %s",
		  oct_type_names[ptr->flags.type]);
	return OCT_ERROR;
    }

    record = NEED_TO_RECORD(ptr, OCT_PUT_POINTS);
    
    gp = (struct geo_points *) ptr;
    
    if (num_points < 0) {
	oct_error("OctPutPoints: Negative number of points not allowed");
	return OCT_ERROR;
    }
    
    old_num_points = gp->num_points;
    old_points = gp->points;
   
    if (old_num_points != 0) {
	oct_bb_get(ptr, &old_box);
    }
    
    if (points == NIL(struct octPoint)) {
	oct_error("OctPutPoints: The point array is null");
	return OCT_ERROR;
    }
    
    pt_ptr = head = (struct octPoint *)
	MALLOC((unsigned) num_points*sizeof(struct octPoint));
    if (pt_ptr == NIL(struct octPoint)) {
	oct_error("OctPutPoints: Out of Memory");
	return OCT_ERROR;
    }
    for (i = num_points; i > 0; i--) {
	*(pt_ptr++) = *(points++);
    }

    gp->points = head;
    gp->num_points = num_points;

        /* Force a recomputation the bounding box */
    MAKE_BB_INVALID(&gp->bbox);
    (*object_desc->bb_func)(ptr, &new_box);
    
    if (old_num_points != 0) {
	if (num_points != 0) {
	    oct_bb_modify(ptr, &old_box, &new_box);
	} else {
	    oct_bb_delete(ptr->facet, &old_box);
	}
    } else if (num_points != 0) {
	oct_bb_add(ptr->facet, &new_box);
    }
    
    if (record) {
	return oct_record_change(ptr, ptr->flags.type, OCT_PUT_POINTS, oct_null_xid, NIL(octObject),
				    old_num_points, old_points, NIL(struct octBox));
    } else if (old_num_points > 0) {
	FREE(old_points);
    }
    return OCT_OK;
}




