#include "copyright.h"
#include "port.h"
#include "oct.h"
#include "oh.h"
#include "tr.h"

octStatus flatten_contacts(facet)
octObject *facet;
{
    octStatus s;
    octObject bag, inst;
    octGenerator gen;
    tr_stack *stack = tr_create();

    s = ohGetByBagName(facet, &bag, "CONNECTORS");
    if (s == OCT_NOT_FOUND) return OCT_OK;
    else if (s != OCT_OK) return s;

    OH_ASSERT(octInitGenContents(&bag, OCT_INSTANCE_MASK, &gen));
    while ((s = octGenerate(&gen, &inst)) == OCT_OK) {
	tr_push(stack);
	tr_add_transform(stack, &inst.contents.instance.transform, 0);
	s = flatten_instance(facet, &inst, stack);
	if (s != OCT_OK) return s;
	tr_pop(stack);
	if ((s = octDelete(&inst)) != OCT_OK) return s;
    }
    tr_free(stack);
    return OCT_OK;
}

octStatus flatten_instance(facet, inst, stack)
octObject *facet;
octObject *inst;
tr_stack *stack;
{
    octObject master, layer, new_layer, geo;
    octGenerator gen, gen1;
    octPoint *point, *pts;
    octStatus s;
    int i;
    int32 num;

    s = ohOpenMaster(&master, inst, "contents", "r");
    if (s < OCT_OK) return s;

    s = octInitGenContents(&master, OCT_LAYER_MASK, &gen);
    while ((s = octGenerate(&gen, &layer)) == OCT_OK) {

	s = ohGetOrCreateLayer(facet, &new_layer, layer.contents.layer.name);
	if (s != OCT_OK) return s;

	s = octInitGenContents(&layer, OCT_GEO_MASK, &gen1);
	while ((s = octGenerate(&gen1, &geo)) == OCT_OK) {
	    switch(geo.type) {
		case OCT_BOX:
		    point = &geo.contents.box.lowerLeft;
		    tr_transform(stack, &(point->x), &(point->y));
		    point = &geo.contents.box.upperRight;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;
		
		case OCT_CIRCLE:	/* better than nothing ... */
		    point = &geo.contents.circle.center;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;

		case OCT_PATH:
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    ohGetPoints(&geo, &num, &pts);
		    for(i = 0; i < num; i++) {
			tr_transform(stack, &(pts[i].x), &(pts[i].y));
		    }
		    if ((s = octPutPoints(&geo, num, pts)) != OCT_OK) return s;
		    break;

		case OCT_POLYGON:
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    ohGetPoints(&geo, &num, &pts);
		    for(i = 0; i < num; i++) {
			tr_transform(stack, &(pts[i].x), &(pts[i].y));
		    }
		    if ((s = octPutPoints(&geo, num, pts)) != OCT_OK) return s;
		    break;

		case OCT_LABEL:
		    point = &geo.contents.label.region.lowerLeft;
		    tr_transform(stack, &(point->x), &(point->y));
		    point = &geo.contents.label.region.upperRight;
		    tr_transform(stack, &(point->x), &(point->y));
		    if ((s = octCreate(&new_layer, &geo)) != OCT_OK) return s;
		    break;
	    }
	}
	if (s != OCT_GEN_DONE) return s;
    }
    if (s != OCT_GEN_DONE) return s;
    return OCT_OK;
}
