/*
 * Symbolic Instances
 *
 * David Harrison 
 * University of California, Berkeley
 * 1990
 *
 * This file contains routines for creating and manipulating symbolic
 * instances.  It includes routines for creating new instances,
 * reconnecting existing instances, and replacing an instance
 * with a new master.
 */

#ifndef _INST_H_
#define _INST_H_

#include "ansi.h"
#include "oct.h"
#include "segments.h"
#include "delete.h"

typedef struct sym_lyr_alt_defn {
    int (*lyr_alt_func)		/* Alternate implementation function */
      ARGS((octObject *t1, octObject *t2, int num_alt, octObject lyrs[],
	    char *data));
    char *data;			/* User data */
} symLyrAltFunc;

extern void symSetLyrAltFunc
  ARGS((symLyrAltFunc *new, symLyrAltFunc *old));

extern int symInstanceP
  ARGS((octObject *obj));
  /* Returns a non-zero status if object is real instance or instance term */

extern void symInstance
  ARGS((octObject *fct, octObject *inst));
  /* Creates a new symbolic instance or connector - no terminal maintenence */

extern void symReconnect
  ARGS((octObject *inst, symConFunc *confun, symDelItemFunc *del_func));
  /* Reconnects and existing instance by examining abutting elements */

extern int symDirChanges
  ARGS((octObject *term, int num));
  /* Returns non-zero if there are atleast `num' changes in directions */

#endif /* _INST_H_ */
