/*
 * Creating new segments in symbolic
 *
 * David Harrison
 * University of California, Berkeley
 * 1989
 *
 * This file contains code for creating new segments in a symbolic
 * cell.  Connectivity information is updated appropriately.  This
 * file contains definitions for using the segment functions.
 */

#ifndef _SEGMENTS_H_
#define _SEGMENTS_H_

#include "ansi.h"
#include "contact.h"

extern void symSegments
  ARGS((octObject *facet, octObject *first, octObject *last,
	int np, struct octPoint *points,
	int nl, octObject layers[],
	int nw, octCoord widths[],
	symConFunc *confun));
  /* Creates a series of new symbolic segments */

extern void symSegChange
  ARGS((octObject *fct, octObject *seg, octObject *new_lyr,
	octCoord new_width, symConFunc *confun));
  /* Changes the layer or width of an existing segment */

extern void symSegDesc
  ARGS((octObject *seg, struct octPoint *ref, tapLayerListElement *elem));
  /* Returns a tap style description of a segment */

extern int symSegConnP
  ARGS((octObject *term, tapLayerListElement *elem));
  /* Returns non-zero if terminal is appropriate for segment description */

extern int symSegBreakP
  ARGS((octObject *seg, tapLayerListElement *elem,
	struct octPoint *isect, symConFunc *confun));
  /* Returns non-zero if `seg' can be broken by `elem' */

#endif /* _SEGMENTS_H_ */
